/*
 * Copyright (c) 2016 Intel Corporation
 *
 * SPDX-License-Identifier: Apache-2.0
 */

#include <zephyr/kernel.h>

#include <zephyr/pm/device.h>
#include <zephyr/pm/pm.h>
#include <nrfx_spim.h>
#include <zephyr/drivers/gpio.h>
#include <hal/nrf_gpio.h>

#include <zephyr/drivers/spi.h>

/* 1000 msec = 1 sec */
#define SLEEP_TIME_MS   1000

/******************************SPIM*******************************************/
#define FLASH_CS_PIN      18  //p0.18
#define FLASH_SCK_PIN     17  //p0.17
#define FLASH_MISO_PIN    14  //p0.14
#define FLASH_MOSI_PIN    13  //p0.13
#define FLASH_MEM_HOLD_PIN DT_ALIAS(flashholdpin)
#define FLASH_MEM_WP_PIN DT_ALIAS(flashwppin)

#define SPI4_INSTANCE 4
static nrfx_spim_t spim4 = NRFX_SPIM_INSTANCE(SPI4_INSTANCE);
nrfx_spim_config_t spim4_config = NRFX_SPIM_DEFAULT_CONFIG(FLASH_SCK_PIN, 
                                  FLASH_MOSI_PIN, FLASH_MISO_PIN, FLASH_CS_PIN);
const struct gpio_dt_spec FLASH_WP_PIN = GPIO_DT_SPEC_GET(FLASH_MEM_WP_PIN, 
                                                                        gpios);
const struct gpio_dt_spec FLASH_HOLD_PIN = GPIO_DT_SPEC_GET(FLASH_MEM_HOLD_PIN, 
                                                                         gpios);

/******************************END*******************************************/


int main(void)
{
	int ret = 0;
	// struct pm_state_info sleep_state = {PM_STATE_SOFT_OFF, 0, 0};

	/***************************FLASH init**************************************/
	const struct device *spi_dev = device_get_binding("SPI_4");
	nrfx_spim_xfer_desc_t pdata;
	spim4_config.mode = NRF_SPIM_MODE_0;
	spim4_config.frequency = NRF_SPIM_FREQ_8M;
	ret = nrfx_spim_init(&spim4,&spim4_config, NULL, NULL);
	ret = gpio_pin_configure_dt(&FLASH_HOLD_PIN,GPIO_OUTPUT_INACTIVE);
	ret = gpio_pin_configure_dt(&FLASH_WP_PIN,GPIO_OUTPUT_INACTIVE);

	uint8_t data  = 0x00;
            
	pdata.p_rx_buffer = NULL;
	pdata.p_tx_buffer = &data;
	pdata.rx_length = 0;
	pdata.tx_length = 1;

	ret = nrfx_spim_xfer(&spim4,&pdata,0);
	/***************************END *****************************************/

	while(1)
	{
		k_msleep(2000);	

		/***************************spim-pm-test**************************/
		ret = pm_device_action_run(spi_dev, PM_DEVICE_ACTION_RESUME);			
		//write to flash
		ret = nrfx_spim_xfer(&spim4,& pdata,0);
		k_msleep(3000);
		ret = pm_device_action_run(spi_dev, PM_DEVICE_ACTION_SUSPEND);
		/*****************************end**********************************/
		

	}

	return 0;
}
